/**
 * Model Base类模块
 */
DGC.Model = $.inherits(jQuery.Observer , {


    /**
     * 唯一标识ID
     * @type String
     */
    id : undefined,

    /**
     * 构造函数
     * @param {} config
     */
    constructor : function(config){
        $.extend(this, config);

        //声明请求超时事件
        this.defineEvents('timeout');

        this.registryMessages();
    },

    /**
     * 子类必须override该方法
     */
    registryMessages: function(){
        //this.service.on(DGC.Service.Messages.XXX, this.fn, this);
    },

    /**
     * 通知UI请求失败
     */
    fireTimeout: function(){
        this.fire('timeout');
    },


    /**
     * 获得可选参数字符串
     * @param {Object} params 参数对象
     * @param {Array} fields 字段
     */
    getOptionsString: function(params, fields){
        var options = [];
        for(var i = 0; i< fields.length; i++){
            var key = fields[i];
            var value = params[key];
            if(value){
                options.push(key + "="+ value);
            }
        }

        return options.length > 0 ? ( "&" + options.join("&") ) : "";
    },


    /**
     * 统一处理响应结果
     * @param {Object} msg 后台响应的结果信息
     * @return {Boolean}  notShowTip 响应成功内容返回对象,响应失败结果返回false
     */
    processResponse: function(msg,notShowTip){
        var DAU = DGC.ActionUtil,
            tipMsg,
            tipParams;
        //不判断HTTP响应状态码
        if(msg.success){

            //如果提交验证失败
            if(msg.data.hasOwnProperty("err_detail")){

                //是否显示错误TIP
                if(!notShowTip){

                    //提示字符串
                    tipMsg = msg.data.err_detail;

                    //字符串参数
                    tipParams = msg.data.params;

                    //存在字符串参数
                    if(tipParams && tipParams.length > 0){
                        DAU.showTipMsg($.format(tipMsg, tipParams));
                    }else{
                        DAU.showTipMsg(tipMsg);
                    }
                }

                return {
                    success: false,
                    data: msg.data
                };
            }else{
                //响应成功获取数据结构
                var mixResponse =  msg.data["result"];

                //遍历所有响应结果，显示错误提示
                for(var method in mixResponse){
                    this.showError(mixResponse[method]);
                }

                return {
                    success : true,
                    data : mixResponse
                }
            }
        }else{
            var msg = msg.data.error;
            DAU.showTipMsg(msg, $.bindFn(this.fireTimeout,this)); //Native拦截alert事件,使用原生方式提示
            return false;
        }
    },

    /**
     * 统一外部API处理流程
     * @param {Object} msg 后台响应的结果信息
     */
    processResponseAPI: function(msg){
        var DAU = DGC.ActionUtil;
        //不判断HTTP响应状态码
        if(msg.success){
            return {
                success : true,
                data : msg.data
            }
        }else{
            var msg = msg.data.error;
            DAU.showTipMsg(msg); //Native拦截alert事件,使用原生方式提示
            return false;
        }
    },

    /**
     * 获得服务端GET响应的对象
     */
    getQueryReponse: function(result){
        return result.data["GET"];
    },

    /*
     * 获得服务端POST响应的对像
     */
    getPostResponse: function(result){
        return result.data["POST"];
    },

    /**
     * 获取缺省请求的信息
     */
    getDefaultPostObj: function(msg){
        return msg.msgInfo.params["POST"][0].data;
    },

    /**
     * 获得用户自定义数据
     */
    getPlugParams: function(msg){
        return msg.msgInfo.plugData;
    },

    /**
     * 某个方法请求的响应集合
     */
    showError: function(action){
        for(var url in action){
            var responseBody = action[url];
            if(responseBody && responseBody.err_detail){
                DGC.ActionUtil.showTipMsg(responseBody.err_detail);
            }
        }
    },

    /**
     * 获得请求的KEY值
     */
    getRequestKey : function(msg){
        var msgObj = msg.msgInfo;
        var key = {
            msgID: msgObj.msgID,
            params: msgObj.params
        }

        if(DEBUG.isWebModel()){
            key.version = msgObj.version;
            key.mockUrl = msgObj.mockUrl;
        }

        return key;
    },

    /**
     * 新增/替换缓存数据
     * @param {String} key 缓存键
     * @param {String|Object} value 缓存值,如果为对象，则自动序列化为JSON
     */
    putCache: function(key, value){
       var cacheItem = $.isString(value) ? value : $.toJSON(value);
       window.sessionStorage.setItem(key, cacheItem);
    },

    /**
     * 使用对象缓存数据(使用序列化后的JSON作为KEY)
     */
    putCacheByObj : function(keyObj,value){
        this.putCache($.toJSON(keyObj),value);
    },

    /**
     * 获取缓存数据
     * @param {String} key 获得缓存值需要的键
     * @param {Boolean}  是否获得纯缓存文本(为true),默认(false)将自动转换为JSON对象
     */
    getCache: function(key){
        var localCache = window.sessionStorage.getItem(key);
        return (arguments.length > 1 && arguments[1]) ? localCache : $.parseJSON(localCache);
    },

    /**
     * 获取缓存数据(使用序列化后的JSON作为KEY)
     */
    getCacheByObj : function(keyObj){
        var args = arguments;
        args[0] = $.toJSON(keyObj);
        return this.getCache.apply(this,args);
    },

    /**
     * 清除缓存数据
     * @param {key} 模糊匹配的Cache Key
     */
    clearCacheByKey: function(key){
        var storage = window.sessionStorage;
        for (var i = 0; i < storage.length; i++) {
           var findKey = storage.key(i);
           if(findKey.indexOf(key) != -1){
               storage.removeItem(findKey);
           }
        };
    }


});
