/**
 * 扩展jQuery实现通用方法
 */
(function($){

    // Shared empty constructor function to aid in prototype-chain creation.
    var emptyClass = function(){};

    /**
     * 类继承实现
     * @param {Function} parent 父类
     * @param {Object}  protoProps 成员变量/方法
     * @param {Object}  staticProps 静态变量/方法
     */
    var inherits = function(parent, protoProps, staticProps) {
        // Helper function to correctly set up the prototype chain, for subclasses.
        // Similar to `goog.inherits`, but uses a hash of prototype properties and
        // class properties to be extended.
        var child;

        // The constructor function for the new subclass is either defined by you
        // (the "constructor" property in your `extend` definition), or defaulted
        // by us to simply call `super()`.
        if (protoProps && protoProps.hasOwnProperty('constructor')) {
          child = protoProps.constructor;
        } else {
          child = function(){ return parent.apply(this, arguments); };
        }

        // Inherit class (static) properties from parent.
        $.extend(true, child, parent);

        // Set the prototype chain to inherit from `parent`, without calling
        // `parent`'s constructor function.
        emptyClass.prototype = parent.prototype;
        child.prototype = new emptyClass();

        // Add prototype properties (instance properties) to the subclass,
        // if supplied.
        if (protoProps) $.extend(true, child.prototype, protoProps);

        // Add static properties to the constructor function, if supplied.
        if (staticProps) $.extend(true, child, staticProps);

        // Correctly set child's `prototype.constructor`.
        child.prototype.constructor = child;

        // Set a convenience property in case the parent's prototype is needed later.
        child.superclass = parent.prototype;

        return child;
    };

    /**
     * 加载图片
     * @param {String} src 图片URL
     * @param {Function} callback 回调函数
     * @param {Object||Any} data 附加属性
     */
    var loadImage = function(src, callback, data){
        var img = new Image(); //创建一个Image对象，实现图片的预下载
        img.src = src;

        if (img.complete) { // 如果图片已经存在于浏览器缓存，直接调用回调函数
            callback.call(img, data);
            return; // 直接返回，不用再处理onload事件
        }

        img.onload = function(){ //图片下载完毕时异步调用callback函数。
            callback.call(img, data); //将回调函数的this替换为Image对象
        };
    }

    /**
     * 委托执行执行函数Fn
     * @param {Function} fn  被委托函数对象
     * @param {Object} obj  函数执行作用域
     * @param {Array} args  执行时的参数
     * @param {Boolean} isAppend 是否为追加参数
     */
    var fnDelegate = function(fn, obj, args ,isAppend){
        if (!$.isFunction(fn)) {
            return fn;
        }

        return function() {
            var callArgs = args || arguments;
            if (isAppend){
                callArgs = Array.prototype.slice.call(arguments, 0);
                callArgs = callArgs.concat(args);
            }
            return fn.apply(obj || window, callArgs);
        };
    }

    /**
     * 字符串占位替换，支持{数字}格式替换
     * @param {String} str 待替换的字符串
     * @param {Array} params 被占位替换的参数
     */
    var format = function(str , params){
        //验证数组
        if( Object.prototype.toString.apply(params) !== '[object Array]' || params.length == 0) {
            return str;
        }
        //支持“{数字}”格式占位替换,数字范围为0-999
        var regx = /\{\d{1,3}\}/g;
        var signs = str.match(regx);
        do{
            var sign = signs.pop(), //获得占位符
                index =  parseInt(sign.match(/\d{1,3}/)[0],10); //获得数组索引
            str = str.replace(sign, params[index]);
        }while(signs.length);

        return str;
    }

    /**
     * 判断是否为空字符串
     */
    var isEmpty = function(value, allowBlank){
        var isNull = value == null,
            blankAllowed = !allowBlank ? value === '' : false;
        return isNull || blankAllowed;
    }

    /**
     * 判断一个对象是否为String类型
     */
    var isString = function(obj){
        return !!(obj === '' || (obj && obj.charCodeAt && obj.substr));
    }

    /**
     * 自定义Observer事件模型
     */
    var Observer = inherits(Object, {

        /**
         * 构造函数
         */
        constructor: function(){
            this._callbacks = {};
        },

        /**
         * 声明自定义事件
         * @param {String} options 一个或多个参数自定义事件名称
         */
        defineEvents: function(options){
            var a = arguments,
                i = a.length,
                calls = this._callbacks =  this._callbacks || {};
            while (i--) {
                 calls[a[i]] = [];
            }
        },

        /**
         * 添加监听自定义事件
         * @param {String} name 自定义事件
         * @param {Function} fn 监听函数
         * @param {Object} context 作用域
         */
        on : function(name, fn, context) {
            var calls = this._callbacks;
            var list = calls[name] || (calls[name] = []);
            list.push([fn, context]);
            return this;
        },


        /**
         * 移除监听自定义事件
         * @param {String} name 自定义事件
         * @param {Function} fn 监听函数
         */
        un : function(name , fn) {
            // Remove one or many callbacks. If `callback` is null, removes all
            // callbacks for the event. If `ev` is null, removes all bound callbacks
            // for all events.
            var calls;
            if (!ev) {
                this._callbacks = {};
            } else if (calls = this._callbacks) {
                if (!fn) {
                    calls[name] = [];
                } else {
                    var list = calls[name];
                    if (!list)
                        return this;
                    for (var i = 0, l = list.length; i < l; i++) {
                        if (list[i] && fn === list[i][0]) {
                            list[i] = null;
                            break;
                        }
                    }
                }
            }
            return this;
        },


        /**
         * 触发自定义事件
         * @param {String} name 自定义事件
         * @param {Object} 触发事件时，需要的参数
         */
        fire : function(name) {
            // Trigger an event, firing all bound callbacks. Callbacks are passed the
            // same arguments as `trigger` is, apart from the event name.
            // Listening for `"all"` passes the true event name as the first argument.
            var list, calls, ev, callback, args;
            var both = 2;
            if (!(calls = this._callbacks))
                return this;
            while (both--) {
                ev = both ? name : 'all';
                if (list = calls[ev]) {
                    for (var i = 0, l = list.length; i < l; i++) {
                        if (!(callback = list[i])) {
                            list.splice(i, 1);
                            i--;
                            l--;
                        } else {
                            args = both ? Array.prototype.slice.call(
                                    arguments, 1) : arguments;
                            callback[0]
                                    .apply(callback[1] || this, args);
                        }
                    }
                }
            }
            return this;
        }
    });


    $.extend({
        /*对象*/
        Observer: Observer,
        /*方法*/
        isString: isString,
        inherits: inherits,
        bindFn: fnDelegate,
        format: format,
        emptyFn: emptyClass,
        isEmpty: isEmpty,
        loadImg: loadImage
    });
})(jQuery);

